# Copyright (c) 2017 Renaud METRICH
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote products
#    derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

package AdvPowerMgr::Plugin::Program::Time;

use strict;
use warnings;

use base qw( AdvPowerMgr::Plugin::GenericText );

use AdvPowerMgr::I18N qw( i18n );
use AdvPowerMgr::Utils qw( list list_splitter uniq );

sub _text_validator_i18n_msg {
    return 'program.unexpected_time_string';
}

our $regex = "(?:[0-1]?[0-9]|2[0-3]):[0-5][0-9]";
our $regex_matcher = "([0-1]?[0-9]|2[0-3]):([0-5][0-9])";

sub text_normalizer {
    my $class = shift;
    my $value = shift;

    my @res = ();

    my @list = list_splitter($regex, $value);

    foreach my $item (@list) {
        $item =~ m#^$regex_matcher$#;
        push(@res, sprintf("%02d:%02d", $1, $2));
    }

    @res = uniq(sort @res);

    return join(",", @res);
}

sub text_trigger {
    my $self = shift;

    my $rest = $self->text();

    my @list = list_splitter($regex, $self->text());
    my @res = ();

    foreach my $item (@list) {
        $item =~ m#$regex_matcher#i;
        push(@res, { h => int($1), m => int($2) });
    }

    $self->_repr(\@res);
}

sub _regex {
    my $class = shift;

    return list($regex);
}

1;

# vi:set autoindent tabstop=8 shiftwidth=4 softtabstop=4 expandtab:
